from jinja2 import Environment
from jinja2 import Template
from jinjaMarkdown.markdownExtension import markdownExtension

environment: Environment = Environment(extensions=[markdownExtension])

def test_bold():
    template: Template = environment.from_string("""{{ "abc" | bold }}""")
    assert template.render() == "abc"
    template: Template = environment.from_string("""{{ "**abc**" | bold }}""")
    assert template.render() == "<strong>abc</strong>"
    template: Template = environment.from_string("""{{ "**bold** again **bold** **bold** again **bold**" | bold }}""")
    assert template.render() == "<strong>bold</strong> again <strong>bold</strong> <strong>bold</strong> again <strong>bold</strong>"
    template: Template = environment.from_string("""{{ "hello **bold** bye wait **bold** again **bold**" | bold }}""")
    assert template.render() == "hello <strong>bold</strong> bye wait <strong>bold</strong> again <strong>bold</strong>"

def test_italics():
    template: Template = environment.from_string("""{{ "abc" | italics }}""")
    assert template.render() == "abc"
    template: Template = environment.from_string("""{{ "*abc*" | italics }}""")
    assert template.render() == "<em>abc</em>"
    template: Template = environment.from_string("""{{ "*italics* again *italics* *italics* again *italics*" | italics }}""")
    assert template.render() == "<em>italics</em> again <em>italics</em> <em>italics</em> again <em>italics</em>"
    template: Template = environment.from_string("""{{ "hello *italics* bye wait *italics* again *italics*" | italics }}""")
    assert template.render() == "hello <em>italics</em> bye wait <em>italics</em> again <em>italics</em>"

def test_boldItalics():
    template: Template = environment.from_string("""{{ "abc" | boldItalics }}""")
    assert template.render() == "abc"
    template: Template = environment.from_string("""{{ "***boldItalics*** again ***boldItalics*** ***boldItalics*** again ***boldItalics***" | boldItalics }}""")
    assert template.render() == "<strong><em>boldItalics</em></strong> again <strong><em>boldItalics</em></strong> <strong><em>boldItalics</em></strong> again <strong><em>boldItalics</em></strong>"
    template: Template = environment.from_string("""{{ "hello ***boldItalics*** bye wait ***boldItalics*** again ***boldItalics***" | boldItalics }}""")
    assert template.render() == "hello <strong><em>boldItalics</em></strong> bye wait <strong><em>boldItalics</em></strong> again <strong><em>boldItalics</em></strong>"

def test_heading():
    template: Template = environment.from_string("""{{ "abc" | heading }}""")
    assert template.render() == "abc"

    template: Template = environment.from_string("""{{ "# abc" | heading }}""")
    assert template.render() == "<h1>abc</h1>"

    template: Template = environment.from_string("""{{ "##### hello bold bye" | heading }}""")
    assert template.render() == "<h5>hello bold bye</h5>"

def test_image():
    template: Template = environment.from_string("""{{ "![city](https://ichef.bbci.co.uk/news/976/cpsprodpb/12A9B/production/_111434467_gettyimages-1143489763.jpg)" | image }}""")
    assert template.render() == "<img src=https://ichef.bbci.co.uk/news/976/cpsprodpb/12A9B/production/_111434467_gettyimages-1143489763.jpg alt=city>"

def test_link():
    template: Template = environment.from_string("""{{ "My favorite search engine is [Duck Duck Go](https://duckduckgo.com)." | link }}""")
    assert template.render() == "My favorite search engine is <a href=https://duckduckgo.com>Duck Duck Go</a>."

def test_lineBreaks():
    template: Template = environment.from_string("""{{ "This is the first line.  \nAnd this is the second line." | lineBreak }}""")
    assert template.render() == "This is the first line.<br>\nAnd this is the second line."

    template: Template = environment.from_string("""{{ "abc  \ndef  \nghi    " | lineBreak }}""")
    assert template.render() == "abc<br>\ndef<br>\nghi<br><br>"

def test_markdown():
    template: Template = environment.from_string("""{{ "abc" | markdown }}""")
    assert template.render() == "<p>abc</p>"

    template: Template = environment.from_string("""{{ "abc\n\ndef" | markdown }}""")
    assert template.render() == "<p>abc</p><p>def</p>"

    template: Template = environment.from_string("""{{ "# abc\n\n**def**" | markdown }}""")
    assert template.render() == "<p><h1>abc</h1></p>" \
                                "<p><strong>def</strong></p>"
    template: Template = environment.from_string("""{{ "I really like using Markdown.\n\nI think I'll use it to format all of my documents from now on." | markdown }}""")
    assert template.render() == "<p>I really like using Markdown.</p><p>I think I'll use it to format all of my documents from now on.</p>"

    template: Template = environment.from_string("""{{ "abc\n\ndef" | markdown }}""")
    assert template.render() == "<p>abc</p><p>def</p>"

    template: Template = environment.from_string("""{{ "***yo*** im writing stuff!  \n*crazy* **right**\n\nAnyways**...**" | markdown }}""")
    assert template.render() == "<p><strong><em>yo</em></strong> im writing stuff!<br>\n<em>crazy</em> <strong>right</strong></p><p>Anyways<strong>...</strong></p>"

    template: Template = environment.from_string("""{{ "*20* <br>## 20" | markdown }}""")
    assert template.render() == "<p><em>20</em> </p><p><h2>20</h2></p>"


    template: Template = environment.from_string(
        """{{ "***hey*** I'm writing stuff!  \n*crazy* **right**\n\nAnyways**...**\n\n![dinosaur!](https://www.example.com/images/dinosaur.jpg)" | markdown }}""")
    assert template.render() == "<p><strong><em>hey</em></strong> I'm writing stuff!<br>\n<em>crazy</em> <strong>right</strong></p><p>Anyways<strong>...</strong></p><p><img src=https://www.example.com/images/dinosaur.jpg alt=dinosaur!></p>"


from jinja2.ext import Extension

def boldItalics(inputText):
    # simple bold markdown command
    outputText = ""
    j = 0
    i = 0
    while i < len(inputText):
        if i < len(inputText) - 2:
            if (inputText[i] == "*" and inputText[i+1] == "*" and inputText[i+2] == "*" and i<len(inputText)-3):
                for j in range(i+3, len(inputText)):
                    if inputText[j]=="*" and inputText[j+1]=="*" and inputText[j+2]=="*":
                        outputText += "<strong><em>" + inputText[i + 3:j] + "</em></strong>"
                        i += 3 + j - i
                        break
            else:
                outputText += inputText[i]
                i += 1
        else:
            outputText += inputText[i]
            i+=1

    return outputText

def bold(inputText):
    # simple bold markdown command
    outputText = ""
    j = 0
    i = 0
    while i < len(inputText):
        if i < len(inputText) - 1:
            if (inputText[i] == "*" and inputText[i+1] == "*" and i<len(inputText)-1):
                for j in range(i+2, len(inputText)):
                    if inputText[j]=="*" and inputText[j+1]=="*":
                        outputText += "<strong>" + inputText[i + 2:j] + "</strong>"
                        i += 2 + j - i
                        break
            else:
                outputText += inputText[i]
                i += 1
        else:
            outputText += inputText[i]
            i+=1

    return outputText

def italics(inputText):
    # simple italics markdown command same as bold
    outputText = ""
    previousSection = 0
    j = 0
    i = 0
    while i < len(inputText):
        if (inputText[i] == "*" ):
            for j in range(i+1, len(inputText)):
                if inputText[j]=="*":
                    outputText += "<em>" + inputText[i + 1:j] + "</em>"
                    i += 1 + j - i
                    break
        else:
            outputText += inputText[i]
            i+=1

    return outputText

def heading(inputText):
    if inputText[0] == "#":
        i = 1
        while inputText[i] != " ":
            i += 1
        outputText = "<h" + str(i) + ">" + inputText[i+1:] + "</h" + str(i) + ">"
        return outputText
    else:
        return inputText

def image(inputText):
    imageAlt = ""
    imageSource =""
    if inputText[0] == "!":
        i = 0
        if inputText[1] == "[":
            i += 2
            while i < len(inputText):
                if inputText[i] == "]":
                    break
                else:
                    imageAlt += inputText[i]
                    i += 1

        if inputText[i+1] == "(":
            j=i + 2
            while j < len(inputText):
                if inputText[j] == ")":
                    break
                else:
                    imageSource += inputText[j]
                    j += 1

        outputText = "<img src=" + imageSource + " alt=" + imageAlt + ">"
        return outputText
    else:
        return  inputText

def link(inputText):
    linkText = ""
    linkAddress = ""
    outputText = ""
    i = 0
    while i < len(inputText)-1:
        if inputText[i] == "[":
            i += 1
            x = i
            while x < len(inputText):
                if inputText[x] == "]":
                    break
                else:
                    linkText += inputText[x]
                    x += 1
            i = x

        elif inputText[i + 1] == "(":
            j = i + 2
            while j < len(inputText):
                if inputText[j] == ")":
                    break
                else:
                    linkAddress += inputText[j]
                    j += 1
            i = j + 1
        else:
            outputText += inputText[i]
            i+=1

    outputText += "<a href=" + linkAddress + ">" + linkText + "</a>"
    outputText += inputText[len(inputText)-1]
    return outputText

def lineBreak(inputText):
    outputText = ""
    previousSection = 0
    stopPharsing = -2
    i = 0
    while i < len(inputText):
        if(i!=len(inputText)-1):
            if inputText[i] == " " and inputText[i+1] == " ":
                outputText += "<br>"
                i+=2
            else:
                outputText += inputText[i]
                i+=1
        else:
            outputText += inputText[i]
            i += 1

    return outputText

def applyMarkdown(inputText):
    outputText = inputText
    #apply image first
    outputText = image(outputText)
    # boldItalics, bold and italics must be ordered from most *** to least *. This is so they are not confused amongst each other
    outputText = boldItalics(outputText)
    outputText = bold(outputText)
    outputText = italics(outputText)

    outputText = heading(outputText)
    outputText = lineBreak(outputText)
    return outputText

def markdown(inputText):
    #actual markdown package
    lineStart = 0
    lineEnd = 0
    currentLine = ""
    outputText = ""
    while lineEnd != len(inputText):
        if inputText[lineEnd:lineEnd + 2]=="\n\n":
            currentLine = inputText[lineStart:lineEnd]
            currentLine = applyMarkdown(currentLine)
            outputText += "<p>" + currentLine + "</p>"
            lineEnd += 2
            lineStart = lineEnd
        if inputText[lineEnd:lineEnd + 4] == "<br>":
            currentLine = inputText[lineStart:lineEnd]
            currentLine = applyMarkdown(currentLine)
            outputText += "<p>" + currentLine + "</p>"
            lineEnd += 4
            lineStart = lineEnd
        lineEnd += 1
    currentLine = inputText[lineStart:lineEnd]
    currentLine = applyMarkdown(currentLine)
    outputText += "<p>" + currentLine + "</p>"

    return outputText






class markdownExtension(Extension):
    def __init__(self, environment):
        super(markdownExtension, self).__init__(environment)
        environment.filters['bold'] = bold
        environment.filters['italics'] = italics
        environment.filters['boldItalics'] = boldItalics
        environment.filters['heading'] = heading
        environment.filters['image'] = image
        environment.filters['link'] = link
        environment.filters['lineBreak'] = lineBreak
        environment.filters['markdown'] = markdown